#!/usr/bin/perl -w
#
# Author: Andrew Robert
# Program: check_queue.pl
#
#
# Version Date       Programmer    Description
# 1.0     7/6/05     AAR           Initial Release


use strict;
use MQSeries;
use MQSeries::Queue;
use MQSeries::Message;
use Getopt::Std;

#
# Variable definitions
#
my $sourceq    = "";
my $qmgr       = "";
my %qatrb;
my $body       = "";
my $subject;
my $retcode;
my $CompCode   = MQCC_FAILED;
my $Reason     = MQRC_UNEXPECTED_ERROR;
my $from       = 'andrew.arobert@gmail.com';
my $sendmail   = "/usr/sbin/sendmail";
my $inq;
my $count=0;

#
# Valid options for inquiry command
#
my @inqlist    = qw/
    AlterationDate  AlterationTime  BackoutRequeueName  BackoutThreshold           
    BaseQName       ClusterDate     ClusterName         ClusterNamelist            
    ClusterQType    ClusterTime     CreationDate        CreationTime               
    CurrentQDepth   DefBind         DefInputOpenOption  DefPersistence             
    DefPriority     DefinitionType  DistLists           HardenGetBackout            
    HighQDepth      InhibitGet      InhibitPut          InitiationQName 
    MaxMsgLength    MaxQDepth       MsgDeliverySequence MsgDeqCount               
    MsgEnqCount     OpenInputCount  OpenOutputCount     ProcessName 
    QDepthHighEvent QDepthHighLimit QDepthLowEvent      QDepthLowLimit    
    QDepthMaxEvent  QDesc           QMgrIdentifier      QMgrName      
    QName           QNames          QServiceInterval    QServiceIntervalEvent
    QType           RemoteQMgrName  RemoteQName         RetentionInterval  
    Scope           Shareability    TimeSinceReset      TriggerControl 
    TriggerData     TriggerDepth    TriggerMsgPriority  TriggerType      
    Usage           XmitQName      
    /;

my @numeric_params = qw/
    AlterationDate      AlterationTime  BackoutThreshold    DefPriority 
    ClusterDate         ClusterTime     CreationDate        CreationTime
    CurrentQDepth       HighQDepth      MaxMsgLength        MaxQDepth 
    MsgDeliverySequence MsgDeqCount     MsgEnqCount         OpenInputCount
    OpenOutputCount     QDepthHighLimit QDepthLowLimit      QMgrIdentifier 
    QServiceInterval    TriggerDepth    TriggerMsgPriority
    /;

#
# Read and assign input from command line or file
#
   


my %opts;
getopts("m:q:t:e:o:f:csh", \%opts);


#
# Display usage if the -h option is specified
#
if (defined $opts{h} )
   {
    print "\n";
    &usage;

   }



#
# Test for minimum number of passed arguments
#
if  (  keys(%opts) < 4 )
    {
     print "\nMissing command line parameters\n\n";
     &usage;

    }


#
# Automatically shift passed values to upper case unless the -c switch is used
#
if ( defined $opts{c} )
  {  
   $sourceq   = $opts{q};  
   $qmgr      = $opts{m};
  }
else
  {
   $sourceq   = uc( $opts{q} );  
   $qmgr      = uc( $opts{m} );
  }
 

#
# See if the option passed is a member of the valid options list. 
# If not, display accepted options list
#  
# The grep command searches based on word boundry and is case sensitive
#
if ( not grep /\b($opts{o})\b/, @inqlist ) 
   {
    print "\nRequested option $opts{o} is not a valid inquiry.";

    #
    # Read through the valid options list and display them to screen in
    # a three column format
    #
    foreach $inq ( @inqlist )
            {
             $inq .= ( " " x ( 22-length($inq) ) );
             
                  if ( $count == 2 )
                     {
                      $inq  .= "\n";
                      $count = 0;
                     }
                  else
                     {
                      $count++;
                     }

            }
            
    print "\nValid options are :\n",@inqlist;
    print "\n";
    exit;
    
   }


#
#Create source queue object 
# linking queue manager and queue to be inquired about 
#
my $sourcequeue = MQSeries::Queue->new
    (
     QueueManager       => $qmgr,
     Queue              => $sourceq,
     CompCode           => $CompCode,
     Reason             => $Reason,
     Options            => MQOO_INQUIRE|MQOO_FAIL_IF_QUIESCING
    )|| die ( "Unable to instantiate $qmgr $sourceq - CompCode => $CompCode, Reason => $Reason\n" ); 



%qatrb = $sourcequeue->Inquire( $opts{o} );

#
# Test and display attribute
#
if ( defined $opts{t} && ( grep /\b($opts{o})\b/, @numeric_params ) )
   {
      if ($qatrb{($opts{o})} >  $opts{t} ) 
         {
          $body     = "\n$opts{o} is $qatrb{($opts{o})}.\n\nThis exceeds the threshold test of $opts{t}\n\n";
          $subject  = "$qmgr :: $sourceq - $opts{o} threshold exceeded  "; 
         }      
      else
         {
          $body    = "\n$opts{o} is $qatrb{($opts{o})}.\n\nThis is below the threshold test of $opts{t}\n\n";
          $subject = "$qmgr :: $sourceq - $opts{o} threshold not exceeded  ";
         }        
   }       
else
   {   
    $body     = "$opts{o} =>  $qatrb{($opts{o})}\n";
    $subject  = "$qmgr :: $sourceq - $opts{o} Report  ";
   }

#
# Test current depth against user defined threshold. If depth exceeds 
# threshold, send warning message via e-mail
#
if ( defined $opts{s} )
   {  
    print $body;  
   }
else
   {
    &mailer( $sendmail, $from, $opts{e}, $subject, $body );
   }



#
# Subroutines
#

#
# Generic mailer
#
sub mailer
    {
    my ( $sendmail, $from, $to, $subject, $body ) = @_;
    open( MAIL, "|$sendmail -oi -t" ) or die "Cannot open $sendmail: $!";
    print MAIL "From: $from\n";
    print MAIL "To: $to\n";
    print MAIL "SubjecT: $subject\n";
    print MAIL "\n$body";
    close MAIL;
   }


#
#
#
#sub readfile
#    {
#     my $filename = shift;
#     my %opts;
#     
#     open(CONF, $filename) or die "Can't read $filename: $!"; 
#  
#     while (<CONF>) 
#           {
#            s/[\r\n]//g;       # Remove windows end-of-line markers
#            s/#.*//g;          # Remove comment lines
#            s/^\s+//;          # Remove extra spaces
#            next unless /\S/;  # Skip blank lines
#
#            my ($k, $v) = split /\s*=\s*/, $_, 2;
#           
#            $config{lc $k} = $v;
#           }
#          
#     close CONF;
#    }


#
#Generic usage statement
#
sub usage
   {
          
    #
    # Strip out the ./ from the beginning of the command 
    #      
    $0 =~ s#.*[/\\]##; 
    
    #
    # Display the text to screen
    #
    print 

<<TEXT;
Usage:   $0 -m {queue manager} -q {queue} -o {inquiry} -e {e-mail address}
    
Optional:  -c  signals script to not upper case passed parameters
Optional:  -t  passes threshold for comparison with -o option. 
               The option must have a numeric value.
Optional:  -s  if used, print to screen instead of mailing result.

Example:  ./$0 -m mqidev1 -q test.aar.q -o CurrentQDepth -t 5 -s 

TEXT

    exit;
   }

